module jpibfi {
    var $ = jQuery;

    export class PinButtonGenerator {

        private _indexer: number;
        private _settings: Settings;
        private _logger: ILogger;

        constructor(settings: Settings, logger: ILogger) {
            this._settings = settings;
            this._logger = logger;
        }
        
        private getImageUrl($elem) : string {
            return ($.fn.prop && $elem.prop('src')) || $elem.attr('src');
        }

        generate(indexer: number): JQuery {
            var $anchor = jQuery('<a/>', {
                href: '#',
                "class": 'pinit-button',
                "data-jpibfi-indexer": indexer,
                text: ""
            });
            
            var that = this;
            $anchor.click(function(){
                that._logger.logString('Pin In button clicked');

                var index = $(this).data("jpibfi-indexer");
                var $image = $('img[data-jpibfi-indexer="' + index + '"]');

                //Bookmark description is created on click because sometimes it's lazy loaded
                var bookmarkDescription = "",
                    descriptionForUrl = "",
                    bookmarkUrl = "";

                //if usePostUrl feature is active, we need to get the data
                if (that._settings.usePostUrl) {
                    var $inputWithData = $image.closest(".jpibfi_container").find("input.jpibfi").first();

                    if ($inputWithData.length) {
                        descriptionForUrl = $inputWithData.data("jpibfi-description");
                        bookmarkUrl = $inputWithData.data("jpibfi-url");
                    }
                }
                bookmarkUrl = bookmarkUrl || that._settings.pageUrl;

                if (that._settings.descriptionOption == 3)
                    bookmarkDescription = $image.attr('title') || $image.attr('alt');
                else if (that._settings.descriptionOption == 2)
                    bookmarkDescription = descriptionForUrl || that._settings.pageDescription;
                else if (that._settings.descriptionOption == 4)
                    bookmarkDescription = that._settings.siteTitle;
                else if (that._settings.descriptionOption == 5)
                    bookmarkDescription = $image.data('jpibfi-description');
                else if (that._settings.descriptionOption == 6)
                    bookmarkDescription = $image.attr('alt'); 

                bookmarkDescription = bookmarkDescription || (descriptionForUrl || that._settings.pageTitle);

                var imageUrl = 'http://pinterest.com/pin/create/bookmarklet/?is_video=' + encodeURIComponent('false')
                    + "&url=" + encodeURIComponent(bookmarkUrl) + "&media=" + encodeURIComponent(that.getImageUrl($image))
                    + '&description=' + encodeURIComponent(bookmarkDescription);

                window.open(imageUrl, 'Pinterest', 'width=632,height=253,status=0,toolbar=0,menubar=0,location=1,scrollbars=1');
                return false;
            });
            
            $anchor.mouseenter(function() {
                var $button = $(this);
                clearTimeout($button.data('jpibfi-timeoutId'));
            });
            
            $anchor.mouseleave(function() {
                var $button = $(this);
                Utils.startHide($button);
            });

            return $anchor;
        }

    }
}