module jpibfi {
    "use strict";
    
    declare var jpibfi_options: any;
    var $ = jQuery;

    class Plugin {

        private _buttonGenerator: PinButtonGenerator;
        private _indexer: number = 0;
        private _logger: Logger;
        private _settings: Settings;

        constructor(options: any) {

            this._settings = new Settings($.extend({
                pageUrl: document.URL,
                pageTitle: document.title,
                pageDescription: $('meta[name="description"]').attr('content') || "",
            }, options));
            this._logger = new Logger(this._settings.debug);
            this._buttonGenerator = new PinButtonGenerator(this._settings, this._logger);

            this._logger.logObject(this._settings);
        }

        addImages() {
            this._logger.logString('>>addImages');

            var $elements = $(this._settings.imageSelector)
                .not(this._settings.notSelector)
                .not('[data-jpibfi-indexer]')
                .filter(this._settings.filterSelector);
            var that = this;
            $elements.each(function() { $(this).attr('data-jpibfi-indexer', that._indexer++); });
            this._logger.logString('Images caught by selectors: ' + this._indexer);
        };

        init() {

            var $containers = $('.jpibfi').closest(this._settings.containerSelector).addClass('jpibfi_container');
            this._logger.logString('Number of containers added: ' + $containers.length);

            this.initImageDelegates();

            $(document).trigger('jpibfi_beforeAddImages', {});
            this.addImages();
            $(document).trigger('jpibfi_afterAddImages', {});
        }

        initImageDelegates() {
            var that = this;
            $(document).delegate('img[data-jpibfi-indexer]', 'mouseenter', function() {
                var $image = $(this);

                if ($image[0].clientWidth < that._settings.minImageWidth || $image[0].clientHeight < that._settings.minImageHeight) {
                    $image.removeAttr('data-jpibfi-indexer');
                    return;
                }

                var indexer = $image.data('jpibfi-indexer');
                var $button = $('a.pinit-button[data-jpibfi-indexer="' + indexer + '"]');

                //button doesn't exist so we need to create it
                if ($button.length == 1){
                    //button exists, we need to clear the timeout that has to remove it
                    clearTimeout($button.data('jpibfi-timeoutId'));
                } else {
                    
                    $button = that._buttonGenerator.generate(indexer);

                    var position = $image.offset();
                    var imageDimensions = {
                        width: $image.get(0).clientWidth,
                        height: $image.get(0).clientHeight
                    };

                    switch (that._settings.buttonPosition) {
                        case ButtonPosition.TopLeft:
                            position.left += that._settings.buttonMarginLeft;
                            position.top += that._settings.buttonMarginTop;
                            break;
                        case ButtonPosition.TopRight:
                            position.top += that._settings.buttonMarginTop;
                            position.left = position.left + imageDimensions.width - that._settings.buttonMarginRight - that._settings.pinImageWidth;
                            break;
                        case ButtonPosition.BottomLeft:
                            position.left += that._settings.buttonMarginLeft;
                            position.top = position.top + imageDimensions.height - that._settings.buttonMarginBottom - that._settings.pinImageHeight;
                            break;
                        case ButtonPosition.BottomRight:
                            position.left = position.left + imageDimensions.width - that._settings.buttonMarginRight - that._settings.pinImageWidth;
                            position.top = position.top + imageDimensions.height - that._settings.buttonMarginBottom - that._settings.pinImageHeight;
                            break;
                        case ButtonPosition.Middle:
                            position.left = Math.round(position.left + imageDimensions.width / 2 - that._settings.pinImageWidth / 2);
                            position.top = Math.round(position.top + imageDimensions.height / 2 - that._settings.pinImageHeight / 2);
                            break;
                    }

                    $image.after($button);
                    $button
                        .show()
                        .offset({ left: position.left, top: position.top });
                }
                $image.addClass('pinit-hover');
            });

            $(document).delegate('img[data-jpibfi-indexer]', 'mouseleave', function() {
                var indexer = $(this).data("jpibfi-indexer");
                var $button = $('a.pinit-button[data-jpibfi-indexer="' + indexer + '"]');
                Utils.startHide($button);
            });
        }
    }

    (function() {
        "use strict";
        
        $(document).ready(function() {
            var plugin = new Plugin(jpibfi_options);
            plugin.init();
        });
    })();
}